#include <inttypes.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/io.h>
#include "player.h"

void player_init(void)
{
	TCCR1A = _BV(COM1A0);	/* OC1A */
	TCCR1B = _BV(WGM12);	/* CTC */

	TCNT1H = 0x00;
	TCNT1L = 0x00;
	OCR1AH = 0x00;
	OCR1AL = 0x00;

	SPK_PORT = SPK_PIN | SPK_MASK;
	SPK_DDR |= SPK_MASK;
}

void player_stop(void)
{
	TCCR1B &= ~(_BV(CS10) | _BV(CS11));
}

void player_start(void)
{
//	TCCR1B |= _BV(CS10) | _BV(CS11);	/* 8 MHz / 64 = 125 kHz */
//	TCCR1B |= _BV(CS11);			/* 8 MHz / 8 = 1 MHz */
	TCCR1B |= _BV(CS10);			/* 8 MHz */
}

void player_play(uint8_t note)
{
#define DIV_H(x) ((uint16_t) (4000000L / x * 4) >> 8)
#define DIV_L(x) ((uint16_t) (4000000L / x * 4) & 0xFF)

	static uint8_t divs_h[] PROGMEM = {
		DIV_H(880.0),	DIV_H(932.4),	DIV_H(987.8),	DIV_H(1046.6),
		DIV_H(1108.8),	DIV_H(1174.8),	DIV_H(1244.8),	DIV_H(1318.8),
		DIV_H(1397.3),	DIV_H(1480.4),	DIV_H(1568.2),	DIV_H(1661.9),

		DIV_H(1760.0),	DIV_H(1864.7),	DIV_H(1975.7),	DIV_H(2093.2),
		DIV_H(2217.7),	DIV_H(2349.7),	DIV_H(2489.5),	DIV_H(2637.7),
		DIV_H(2794.6),	DIV_H(2960.8),	DIV_H(3137.1),	DIV_H(3323.7),
	};

	static uint8_t divs_l[] PROGMEM = {
		DIV_L(880.0),	DIV_L(932.4),	DIV_L(987.8),	DIV_L(1046.6),
		DIV_L(1108.8),	DIV_L(1174.8),	DIV_L(1244.8),	DIV_L(1318.8),
		DIV_L(1397.3),	DIV_L(1480.4),	DIV_L(1568.2),	DIV_L(1661.9),

		DIV_L(1760.0),	DIV_L(1864.7),	DIV_L(1975.7),	DIV_L(2093.2),
		DIV_L(2217.7),	DIV_L(2349.7),	DIV_L(2489.5),	DIV_L(2637.7),
		DIV_L(2794.6),	DIV_L(2960.8),	DIV_L(3137.1),	DIV_L(3323.7),
	};

#undef DIV_L
#undef DIV_H

	OCR1AH = pgm_read_byte_near(divs_h + note);
	OCR1AL = pgm_read_byte_near(divs_l + note);
}
